/*
 *+--------------------------------------------------------------------------------------+
 *| Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and |
 *| conditions in the IBM International Program License Agreement.						 |	
 *|  Copyright IBM Corporation 2003, 2004. All Rights Reserved.						 |
 *+--------------------------------------------------------------------------------------+
 */

package com.ibm.jps.umltoxml.util;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.util.StringTokenizer;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.codegen.jet.JETEmitter;
import org.eclipse.emf.codegen.jet.JETException;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.Model;
import org.eclipse.uml2.Package;

/**
 * An utility class to contain methods to manipulate Eclipse resources.
 */
public class ResourceUtility {

    /**
     * Constructor. Being a utility class with all static methods and no state
     * of its own, this class does not need its constaructor to be public.
     */
    public ResourceUtility() {
        super();
    }

    /**
     * Returns the handle to a file in a folder inside a project. The file or
     * the folder may not yet exist.
     * 
     * Given a project this method creates and returns a handle to a identified
     * file. The folder or the file may not yet exist. If the identified folder
     * does not exist, the method creates the folder under the project, and then
     * creates a file handle to the file.
     * 
     * @param project
     *            Handle to the project containing the folder,
     *            <code>folderName</code> and the file <code>fileName</code>
     * @param folderName
     *            The name of the folder containing the file
     *            <code>filename</code>
     * @param fileName
     *            The name of the file
     * @return A handle to the file. If the file does not exist in the folder
     *         returnvalue.exists() will false.
     * @throws CoreException
     *             If the creation of the folder (in the case it does not exist)
     *             fails
     */
    public static IFile getFileFromProjectFolder(IProject project, String folderName,
            String fileName) throws CoreException {
        IFolder folder = project.getFolder(folderName);
        if (!folder.exists()) {
            folder.create(true, true, new NullProgressMonitor());
        }
        IFile file = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(
                new Path(folder.getLocation() + "/" + fileName));//$NON-NLS-1$
        return file;
    }

    /**
     * Writes the contents of a Writer to a specified file in a specified folder
     * in a specified project. If the file exits, it overwrites the contents
     * only if the flag, overwrite, is true.
     * 
     * @param writer
     *            Source of the file content
     * @param project
     *            Handle to the project containing the folder,
     *            <code>folderName</code> and the file <code>fileName</code>
     * @param folderName
     *            The name of the folder containing the file
     *            <code>filename</code>
     * @param fileName
     *            The name of the file
     * @param overwrite
     *            Flag to indicate if an existing file should be over-written.
     * @throws CoreException
     */
    public static void writeToProjectFile(Writer writer, IProject project, String folderName,
            String fileName, boolean overwrite) throws CoreException {
        InputStream istream = new ByteArrayInputStream(writer.toString().getBytes());
        
        IFolder folder = ensureFolderExists(project, folderName);

        IFile file = getFileFromProjectFolder(project, folderName, fileName);
        if (!file.exists()) {
            file.create(istream, true, new NullProgressMonitor());
        } else if (overwrite) {
            file.setContents(istream, true, true, new NullProgressMonitor());
        }
    }
    
	/**
	 * Create the specified folder in the file system if it does not exist yet.
	 * If it does exist, then no action is taken.
	 * 
	 * @param project is the handle to the project containing the folder,
	 *            <code>folderName</code>.
	 * @param folderName is the name of the folder.
	 * @return A handle to the folder. If the folder does not exist, it will be created
	 * @throws CoreException If the creation of the folder (in the case it does not exist)
	 *             fails
	 */
	private static IFolder ensureFolderExists(IProject project,
			String folderName) throws CoreException {
		IFolder folder = project.getFolder(folderName);
		if (!folder.exists()) {
			// make sure each subfolder exists as well as the full folder name
			// ensure that the folderName uses the File.separator
			folderName = folderName.replace('/', File.separator.charAt(0));
			folderName = folderName.replace('\\', File.separator.charAt(0));
			StringTokenizer strTok = new StringTokenizer(folderName,
					File.separator);
			String subFolderName = "";
			while (strTok.hasMoreTokens()) {
				subFolderName = subFolderName + strTok.nextToken();
				IFolder subFolder = project.getFolder(subFolderName);
				if (!subFolder.exists()) {
					subFolder.create(true, true, new NullProgressMonitor());
				}
				subFolderName += java.io.File.separator;
			}
			folder = project.getFolder(folderName);
			if (!folder.exists()) {
				// user must not have used the File.separator as subdir separator.
				// Ensure that the folderName uses the File.separator
				folderName = folderName.replace('/', File.separator.charAt(0));
				folderName = folderName.replace('\\', File.separator.charAt(0));
				strTok = new StringTokenizer(folderName, File.separator);
				subFolderName = "";
				while (strTok.hasMoreTokens()) {
					subFolderName = subFolderName + strTok.nextToken();
					IFolder subFolder = project.getFolder(subFolderName);
					if (!subFolder.exists()) {
					    System.out.println(">>> Creating subfolder " + subFolderName);
						subFolder.create(true, true, new NullProgressMonitor());
					}
					subFolderName += java.io.File.separator;
				}
				folder = project.getFolder(folderName);
			}
		}
		return folder;
	}

    /**
     * Gets the project a model element is part of.
     * 
     * @param element
     *            The model element. Checks assertion that <code>element</code>
     *            is not null
     * @return A handle to the project containing the model element
     */
    public static IProject getProject(EObject element) {
        URI uri = element.eResource().getURI();
        IFile modelFile = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(
                new Path(uri.device() + uri.path()));
        return modelFile.getProject();
    }

    public static String getPath(Package pkg) {
        String thePath = "";
        if (pkg != null) {
            Package nestingPackage = pkg.getNestingPackage();
            if ((nestingPackage != null) && (!(nestingPackage instanceof Model))) {
                //nestingPackage.
                thePath = File.separator + nestingPackage.getName();
                //	check to see if it has a parent package
                thePath += getPath(nestingPackage);
             }
        }
        return thePath;
    }

    public static void copyFile(String from, String to) throws IOException {
        File inputFile = new File(from);
        File outputFile = new File(to);

        FileInputStream in = new FileInputStream(inputFile);
        FileOutputStream out = new FileOutputStream(outputFile);
        int c;

        while ((c = in.read()) != -1)
           out.write(c);

        in.close();
        out.close();        
    }
    
    public static void copyToProjectFile(String sourceFileName, IProject project, String folderName,
            String fileName, boolean overwrite) throws CoreException, FileNotFoundException {
        File inputFile = new File(sourceFileName);
        FileInputStream istream = new FileInputStream(inputFile);
        
        IFolder folder = ensureFolderExists(project, folderName);

        IFile file = getFileFromProjectFolder(project, folderName, fileName);
        if (!file.exists()) {
            file.create(istream, true, new NullProgressMonitor());
        } else if (overwrite) {
            file.setContents(istream, true, true, new NullProgressMonitor());
        }
    }
    
    public static void generate(Object argument, JETEmitter emitter, IProject project, String folderName,
            String fileName, boolean overwrite) throws IOException, CoreException {
        System.out.println("    Building " + folderName + File.separatorChar + fileName);
        Writer writer = new StringWriter();
        String output = emitter.generate(new NullProgressMonitor(), new Object[] {argument});
        writer.write(output);
        writeToProjectFile(writer, project, folderName, fileName, overwrite);
    }

    public static JETEmitter buildEmitter(String jetTemplate) throws JETException {
        String pluginId = "com.ibm.jps.umltoxml";
        String base = Platform.getBundle(pluginId).getEntry("/").toString();
        
        String uri = base + jetTemplate;
        JETEmitter emitter = new JETEmitter(uri) {
        };
        emitter.addVariable("UML2", "org.eclipse.uml2");
        emitter.addVariable("EMF_COMMON", "org.eclipse.emf.common");
        emitter.addVariable("EMF_ECORE", "org.eclipse.emf.ecore");
        emitter.addVariable("UMLTOXML", "com.ibm.jps.umltoxml");
        //emitter.addVariable("UMLTOXML", pluginId);
        return emitter;
    }

}